//+------------------------------------------------------------------+
//|                                         Confirm14_03 Monday2.mq4 |
//|                                                               JN |
//|                                                josef.noll@gmx.at |
//+------------------------------------------------------------------+
#property copyright "JN"
#property link      "josef.noll@gmx.at"



//+------------------------------------------------------------------+
//|                                                     Confirm14_03 |
//+------------------------------------------------------------------+

/*
Difference between ver 02 and 03 -- the MinMiniEquity and MinStdEquity
settings have been removed.

Attach to any chart, any timeframe. Do not attach to more than one
chart.

We selected two groups of 7 currency pairs (total of 14) that are
mathematically equivalent, and we initially buy one set of 7 and sell
the other set of 7.  We arbitrarily picked one set of pairs to be the
buy side, and the other to be the sell side, and always initally trade
each group in its designated direciton.

When the biggest-loser currency pair has risen/fallen above/below its
24-hour high/low, we use selected pairs to confirm currency weakness
by checking that the confirming pairs have also breached 24-hour
highs/lows in the appropriate direction, and we cut-reverse the
biggest-loser currency pair.

These are the pairs we trade:

For InterbankFX
===============
Group 1 (The Buys)
------------------
1.  GBPUSD, confirming pairs: GBPJPY, GBPCHF, EURGBP
2.  EURGBP, confirming pairs: EURJPY, EURCHF
3.  GBPJPY, confirming pairs: EURGBP, GBPCHF
4.  USDCHF, confirming pairs: EURCHF, GBPCHF
5.  NZDUSD, confirming pair:  NZDJPY
6.  AUDJPY, confirming pairs: CHFJPY, EURJPY
7.  EURJPY, confirming pairs: EURGBP, EURCHF

Group 2 (The Sells)
-------------------
8.  EURUSD, confirming pairs: EURJPY, EURGBP, EURCHF
9.  USDJPY, confirming pairs: CHFJPY, NZDJPY
10. AUDUSD, confirming pair:  AUDJPY
11. NZDJPY, confirming pairs: CHFJPY, EURJPY
12. GBPCHF, confirming pairs: EURGBP, GBPJPY
13. CHFJPY, confirming pairs: EURCHF, GBPCHF
14. EURCHF, confirming pairs: EURJPY, EURGBP

We close all trades when current equity exceeds 1% of last flat
equity.

*/

//+------------------------------------------------------------------+
//|              External Variables                                  |
//+------------------------------------------------------------------+

extern bool   Bailout = false;

extern int    Slippage         =       3;
extern int    MiniSize         =   10000;
extern int    StdSize          =  100000;
extern int    MaxTrades        =      14;

extern int Sunday    = 0;
extern int Monday    = 1;
extern int Tuesday   = 2;
extern int Wednesday = 3;
extern int Thursday  = 4;
extern int Friday    = 5;
extern int Saturday  = 6;

extern bool   UserTradingAllowed   = true;
extern bool   StopAfterNoTrades    = false;
extern bool   UseSmallestLot       = false;

extern double MaxMarginToUse      =     1.0; // Maximum percent of margin to commit to trades.
extern double Lots                =     0.1;
extern double ProfitPct           =     1.0;
extern double MaxMiniProfit       =  1000.0;
extern double MaxStdProfit        = 10000.0;

extern int    MagicNumber         =  375544;


//+------------------------------------------------------------------+
//|              Internal Variables                                  |
//+------------------------------------------------------------------+

int i;
int Leverage;
int NumBuys, NumSells;
int TotalTrades;
int AccountTypeVal;

int GBPUSDbuys, GBPUSDsells;
int EURGBPbuys, EURGBPsells;
int GBPJPYbuys, GBPJPYsells;
int USDCHFbuys, USDCHFsells;
int NZDUSDbuys, NZDUSDsells;
int AUDJPYbuys, AUDJPYsells;
int EURJPYbuys, EURJPYsells;

int EURUSDbuys, EURUSDsells;
int USDJPYbuys, USDJPYsells;
int AUDUSDbuys, AUDUSDsells;
int NZDJPYbuys, NZDJPYsells;
int GBPCHFbuys, GBPCHFsells;
int CHFJPYbuys, CHFJPYsells;
int EURCHFbuys, EURCHFsells;

bool   AccountIsMini;
bool   CloseNextLoser;
bool   CloseAll;
bool   BidsHaveChanged = false;

bool GBPUSDrose;
bool GBPUSDsank;
bool EURGBProse;
bool EURGBPsank;
bool GBPJPYrose;
bool GBPJPYsank;
bool USDCHFrose;
bool USDCHFsank;
bool NZDUSDrose;
bool NZDUSDsank;
bool AUDJPYrose;
bool AUDJPYsank;
bool EURJPYrose;
bool EURJPYsank;

bool EURUSDrose;
bool EURUSDsank;
bool USDJPYrose;
bool USDJPYsank;
bool AUDUSDrose;
bool AUDUSDsank;
bool NZDJPYrose;
bool NZDJPYsank;
bool GBPCHFrose;
bool GBPCHFsank;
bool CHFJPYrose;
bool CHFJPYsank;
bool EURCHFrose;
bool EURCHFsank;

string AccountTypeString;
string LastFlatBal;           double valLastFlatBal;
string LastFlatEq;            double valLastFlatEq;
string LowestMarginLevel;     double valLowestMarginLevel;
string LargestFloatingLoss;   double valLargestFloatingLoss;
string MaxFloatingDrawdown;   double valMaxFloatingDrawdown;

string GBPUSDsym;
string EURGBPsym;
string GBPJPYsym;
string USDCHFsym;
string NZDUSDsym;
string AUDJPYsym;
string EURJPYsym;

string EURUSDsym;
string USDJPYsym;
string AUDUSDsym;
string NZDJPYsym;
string GBPCHFsym;
string CHFJPYsym;
string EURCHFsym;

string CommentString;

double MaxLots;
double MinLots;
double UseLots, TotalLots, BuyLots, SellLots;
double PairOverallPL, EAPL, BuyPL, SellPL, EquityExit;
double PercentMarginLevel;
double DollarProfitTarget;
double MaxMarginVar;
double Investment;

double GBPUSDPL;
double EURGBPPL;
double GBPJPYPL;
double USDCHFPL;
double NZDUSDPL;
double AUDJPYPL;
double EURJPYPL;

double EURUSDPL;
double USDJPYPL;
double AUDUSDPL;
double NZDJPYPL;
double GBPCHFPL;
double CHFJPYPL;
double EURCHFPL;

double prevGBPUSDbid = 0.0;
double prevEURGBPbid = 0.0;
double prevGBPJPYbid = 0.0;
double prevUSDCHFbid = 0.0;
double prevNZDUSDbid = 0.0;
double prevAUDJPYbid = 0.0;
double prevEURJPYbid = 0.0;

double prevEURUSDbid = 0.0;
double prevUSDJPYbid = 0.0;
double prevAUDUSDbid = 0.0;
double prevNZDJPYbid = 0.0;
double prevGBPCHFbid = 0.0;
double prevCHFJPYbid = 0.0;
double prevEURCHFbid = 0.0;

double BiggestWinnerPL;
double BiggestWinnerLots;
int    BiggestWinnerTicket;
string BiggestWinnerType;
string BiggestWinnerSymbol;
double BiggestWinnerPlusBiggestLoser;

double BiggestLoserPL;
double BiggestLoserLots;
int    BiggestLoserTicket;
string BiggestLoserType;
string BiggestLoserSymbol;

//+------------------------------------------------------------------+
//|   Expert Advisor Initialization, executes only when the EA is    |
//|   first attached to a chart or on platform  restart.             |
//+------------------------------------------------------------------+

int init()
{
  SetGlobalVariableNames();
  InitGlobalVars();

  return (0);
}

//+------------------------------------------------------------------+
//|    Expert Advisor start function, executes on each tick of the   |
//|    currecy pair it's attached to.                                |
//+------------------------------------------------------------------+

int start()
{
  if (Bailout || (!IsTesting() && !IsExpertEnabled()))
  {
    Comment("Expert is not enabled");

    return(0);
  }

  RefreshGlobalVars();
  GetAccountInfo();  
  SetSymbols();
  SetAccountType();
  SetLotSize();  
  Count_PL_Trades_Lots();
  SetExitTargets();

  BidsHaveChanged = 

    prevGBPUSDbid != 0.0 &&
    prevEURGBPbid != 0.0 &&
    prevGBPJPYbid != 0.0 &&
    prevUSDCHFbid != 0.0 &&
    prevNZDUSDbid != 0.0 &&
    prevAUDJPYbid != 0.0 &&
    prevEURJPYbid != 0.0 &&

    prevEURUSDbid != 0.0 &&
    prevUSDJPYbid != 0.0 &&
    prevAUDUSDbid != 0.0 &&
    prevNZDJPYbid != 0.0 &&
    prevGBPCHFbid != 0.0 &&
    prevCHFJPYbid != 0.0 &&
    prevEURCHFbid != 0.0 &&

    (prevGBPUSDbid != MarketInfo(GBPUSDsym, MODE_BID) ||
     prevEURGBPbid != MarketInfo(EURGBPsym, MODE_BID) ||
     prevGBPJPYbid != MarketInfo(GBPJPYsym, MODE_BID) ||
     prevUSDCHFbid != MarketInfo(USDCHFsym, MODE_BID) ||
     prevNZDUSDbid != MarketInfo(NZDUSDsym, MODE_BID) ||
     prevAUDJPYbid != MarketInfo(AUDJPYsym, MODE_BID) ||
     prevEURJPYbid != MarketInfo(EURJPYsym, MODE_BID) ||
     prevEURUSDbid != MarketInfo(EURUSDsym, MODE_BID) ||
     prevUSDJPYbid != MarketInfo(USDJPYsym, MODE_BID) ||
     prevAUDUSDbid != MarketInfo(AUDUSDsym, MODE_BID) ||
     prevNZDJPYbid != MarketInfo(NZDJPYsym, MODE_BID) ||
     prevGBPCHFbid != MarketInfo(GBPCHFsym, MODE_BID) ||
     prevCHFJPYbid != MarketInfo(CHFJPYsym, MODE_BID) ||
     prevEURCHFbid != MarketInfo(EURCHFsym, MODE_BID));

  CloseAllTradesIfNeeded();
  TrimIfNeeded();

  if (TotalTrades == 0)
  {
    if (AccountBalance() > valLastFlatBal)
    {
      valLastFlatBal = AccountBalance();
      GlobalVariableSet(LastFlatBal, valLastFlatBal);
    }

    if (AccountEquity() > valLastFlatEq)
    {
      valLastFlatEq = AccountEquity();
      GlobalVariableSet(LastFlatEq,  valLastFlatEq);
    }

    if (!IsExpertEnabled() || StopAfterNoTrades) return(0);

    CloseAll = false;
  }

  if (!CloseAll && valLastFlatEq != 0.0 && AccountEquity()-valLastFlatEq > DollarProfitTarget)
  {
    CloseAll = true; 
    return(0);
  }

  OpenTradesIfNeeded();
  ExamineMarginAndFloat();

  prevGBPUSDbid = MarketInfo(GBPUSDsym, MODE_BID);
  prevEURGBPbid = MarketInfo(EURGBPsym, MODE_BID);
  prevGBPJPYbid = MarketInfo(GBPJPYsym, MODE_BID);
  prevUSDCHFbid = MarketInfo(USDCHFsym, MODE_BID);
  prevNZDUSDbid = MarketInfo(NZDUSDsym, MODE_BID);
  prevAUDJPYbid = MarketInfo(AUDJPYsym, MODE_BID);
  prevEURJPYbid = MarketInfo(EURJPYsym, MODE_BID);

  prevEURUSDbid = MarketInfo(EURUSDsym, MODE_BID);
  prevUSDJPYbid = MarketInfo(USDJPYsym, MODE_BID);
  prevAUDUSDbid = MarketInfo(AUDUSDsym, MODE_BID);
  prevNZDJPYbid = MarketInfo(NZDJPYsym, MODE_BID);
  prevGBPCHFbid = MarketInfo(GBPCHFsym, MODE_BID);
  prevCHFJPYbid = MarketInfo(CHFJPYsym, MODE_BID);
  prevEURCHFbid = MarketInfo(EURCHFsym, MODE_BID);

  PrintChartComments();
    
  return(0);
}

//+------------------------------------------------------------------+
//|          Assign values to global variable namestrings.           |
//+------------------------------------------------------------------+

void SetGlobalVariableNames()
{
  LowestMarginLevel = MagicNumber+"_LowestMarginLevel";
  LargestFloatingLoss = MagicNumber+"_LargestFloatingLoss";
  MaxFloatingDrawdown = MagicNumber+"_MaxFloatingDrawdown";

  LastFlatBal = MagicNumber+"_LastFlatBal";
  LastFlatEq  = MagicNumber+"_LastFlatEq";

  return(0);
}

//+------------------------------------------------------------------+
//|            Assign initial values to global variables.            |
//+------------------------------------------------------------------+

void InitGlobalVars()
{
  if (!GlobalVariableCheck(LowestMarginLevel)) GlobalVariableSet(LowestMarginLevel, 100000000.0);
  if (!GlobalVariableCheck(LargestFloatingLoss)) GlobalVariableSet(LargestFloatingLoss, 0.0);
  if (!GlobalVariableCheck(MaxFloatingDrawdown)) GlobalVariableSet(MaxFloatingDrawdown, 0.0);

  if (!GlobalVariableCheck(LastFlatBal)) GlobalVariableSet(LastFlatBal, AccountBalance());
  if (!GlobalVariableCheck(LastFlatEq)) GlobalVariableSet(LastFlatEq, AccountEquity());

  return(0);
}

//+------------------------------------------------------------------+
//|   Get global variable values via their namestring and assign the |
//|   stored values to the variables used in the EA's logic.         |
//+------------------------------------------------------------------+

void RefreshGlobalVars()
{
  valLowestMarginLevel = GlobalVariableGet(LowestMarginLevel);
  valLargestFloatingLoss = GlobalVariableGet(LargestFloatingLoss);
  valMaxFloatingDrawdown = GlobalVariableGet(MaxFloatingDrawdown);

  valLastFlatBal = GlobalVariableGet(LastFlatBal);
  valLastFlatEq  = GlobalVariableGet(LastFlatEq);

  return(0);
}

//+------------------------------------------------------------------+
//|  Get important account parameters that affect how the lot size   |
//|  will be set.                                                    |
//+------------------------------------------------------------------+

void GetAccountInfo()
{
  // Get account leverage, account type.
  Leverage       = AccountLeverage();

  AccountTypeVal = MarketInfo(Symbol(), MODE_LOTSIZE);
  MaxLots        = MarketInfo(Symbol(), MODE_MAXLOT);
  MinLots        = MarketInfo(Symbol(), MODE_MINLOT);

  return(0);
}

//+------------------------------------------------------------------+
//|                 Set the traded symbol strings.                   |
//+------------------------------------------------------------------+

void SetSymbols()
{
  string AddMangle = "";
  if (StringLen(Symbol()) > 6) AddMangle = StringSubstr(Symbol(), 6);

  GBPUSDsym = "GBPUSD" + AddMangle;
  EURGBPsym = "EURGBP" + AddMangle;
  GBPJPYsym = "GBPJPY" + AddMangle;
  USDCHFsym = "USDCHF" + AddMangle;
  NZDUSDsym = "NZDUSD" + AddMangle;
  AUDJPYsym = "AUDJPY" + AddMangle;
  EURJPYsym = "EURJPY" + AddMangle;

  EURUSDsym = "EURUSD" + AddMangle;
  USDJPYsym = "USDJPY" + AddMangle;
  AUDUSDsym = "AUDUSD" + AddMangle;
  NZDJPYsym = "NZDJPY" + AddMangle;
  GBPCHFsym = "GBPCHF" + AddMangle;
  CHFJPYsym = "CHFJPY" + AddMangle;
  EURCHFsym = "EURCHF" + AddMangle;

  return(0);
}

//+------------------------------------------------------------------+
//|             Set the account type and minimum equity.             |
//+------------------------------------------------------------------+

void SetAccountType()
{
  if (AccountTypeVal == MiniSize || StringLen(Symbol()) == 7)
  {
    AccountIsMini     = true;
    AccountTypeString = "MINI";
  }
  else if (AccountTypeVal == StdSize || StringLen(Symbol()) == 6)
  {
    AccountIsMini     = false;
    AccountTypeString = "STANDARD";
  }
  
  return(0);
}

//+------------------------------------------------------------------+
//|                 Set the lot size for any trades.                 |
//+------------------------------------------------------------------+

void SetLotSize()
{
  Investment = MaxMarginToUse / 100.0;

  // The symbol the EA is attached to should be good enough for the purposes of LOTSIZE
  UseLots = valLastFlatEq * Investment * (Leverage / MarketInfo(Symbol(), MODE_LOTSIZE));
  UseLots = StrToDouble(DoubleToStr(UseLots,2));

  if (UseSmallestLot || UseLots < MinLots) UseLots = MinLots;
  if (UseLots > MaxLots) UseLots = MaxLots;

  return(0);
}

//+------------------------------------------------------------------+
//|  Do some accounting -- count the profit/loss, the open trades,   |
//|  and the number of open lots.                                    |
//+------------------------------------------------------------------+

void Count_PL_Trades_Lots()
{
  NumBuys            =    0;
  NumSells           =    0;
  TotalTrades        =    0;
  EAPL               =    0.0;
  BuyPL              =    0.0;
  SellPL             =    0.0;
  TotalLots          =    0.0;
  BuyLots            =    0.0;
  SellLots           =    0.0;

  GBPUSDbuys = 0; GBPUSDsells = 0;
  EURGBPbuys = 0; EURGBPsells = 0;
  GBPJPYbuys = 0; GBPJPYsells = 0;
  USDCHFbuys = 0; USDCHFsells = 0;
  NZDUSDbuys = 0; NZDUSDsells = 0;
  AUDJPYbuys = 0; AUDJPYsells = 0;
  EURJPYbuys = 0; EURJPYsells = 0;

  EURUSDbuys = 0; EURUSDsells = 0;
  USDJPYbuys = 0; USDJPYsells = 0;
  AUDUSDbuys = 0; AUDUSDsells = 0;
  NZDJPYbuys = 0; NZDJPYsells = 0;
  GBPCHFbuys = 0; GBPCHFsells = 0;
  CHFJPYbuys = 0; CHFJPYsells = 0;
  EURCHFbuys = 0; EURCHFsells = 0;

  BiggestWinnerPL = 0.0;
  BiggestWinnerLots = 0.0;
  BiggestWinnerTicket = 0;
  BiggestWinnerType = "";
  BiggestWinnerSymbol = "";

  BiggestLoserPL = 0.0;
  BiggestLoserLots = 0.0;
  BiggestLoserTicket = 0;
  BiggestLoserType = "";
  BiggestLoserSymbol = "";

  for (i = 0; i < OrdersTotal(); i++)
  {
    OrderSelect(i,SELECT_BY_POS,MODE_TRADES);

    if (OrderMagicNumber() == MagicNumber)
    {
      PairOverallPL = OrderProfit() + OrderSwap() + OrderCommission();
      EAPL  += PairOverallPL;
      TotalLots += OrderLots();

      if (OrderType() == OP_BUY)
      {
        BuyPL += PairOverallPL;

        if (PairOverallPL > BiggestWinnerPL)
        {
          BiggestWinnerPL     = PairOverallPL;
          BiggestWinnerLots   = OrderLots();
          BiggestWinnerTicket = OrderTicket();
          BiggestWinnerType   = "BUY";
          BiggestWinnerSymbol = OrderSymbol();
        }

        if (PairOverallPL < BiggestLoserPL)
        {
          BiggestLoserPL     = PairOverallPL;
          BiggestLoserLots   = OrderLots();
          BiggestLoserTicket = OrderTicket();
          BiggestLoserType   = "BUY";
          BiggestLoserSymbol = OrderSymbol();
        }

        BuyLots += OrderLots();

        NumBuys++;

        if (OrderSymbol() == GBPUSDsym) { GBPUSDbuys++; GBPUSDPL = PairOverallPL; }
        else
        if (OrderSymbol() == EURGBPsym) { EURGBPbuys++; EURGBPPL = PairOverallPL; }
        else
        if (OrderSymbol() == GBPJPYsym) { GBPJPYbuys++; GBPJPYPL = PairOverallPL; }
        else
        if (OrderSymbol() == USDCHFsym) { USDCHFbuys++; USDCHFPL = PairOverallPL; }
        else
        if (OrderSymbol() == NZDUSDsym) { NZDUSDbuys++; NZDUSDPL = PairOverallPL; }
        else
        if (OrderSymbol() == AUDJPYsym) { AUDJPYbuys++; AUDJPYPL = PairOverallPL; }
        else
        if (OrderSymbol() == EURJPYsym) { EURJPYbuys++; EURJPYPL = PairOverallPL; }
        else
        if (OrderSymbol() == EURUSDsym) { EURUSDbuys++; EURUSDPL = PairOverallPL; }
        else
        if (OrderSymbol() == USDJPYsym) { USDJPYbuys++; USDJPYPL = PairOverallPL; }
        else
        if (OrderSymbol() == AUDUSDsym) { AUDUSDbuys++; AUDUSDPL = PairOverallPL; }
        else
        if (OrderSymbol() == NZDJPYsym) { NZDJPYbuys++; NZDJPYPL = PairOverallPL; }
        else
        if (OrderSymbol() == GBPCHFsym) { GBPCHFbuys++; GBPCHFPL = PairOverallPL; }
        else
        if (OrderSymbol() == CHFJPYsym) { CHFJPYbuys++; CHFJPYPL = PairOverallPL; }
        else
        if (OrderSymbol() == EURCHFsym) { EURCHFbuys++; EURCHFPL = PairOverallPL; }
      }

      if (OrderType() == OP_SELL)
      {
        SellPL += PairOverallPL;

        if (PairOverallPL> BiggestWinnerPL)
        {
          BiggestWinnerPL     = PairOverallPL;
          BiggestWinnerLots   = OrderLots();
          BiggestWinnerTicket = OrderTicket();
          BiggestWinnerType   = "SELL";
          BiggestWinnerSymbol = OrderSymbol();
        }

        if (PairOverallPL< BiggestLoserPL)
        {
          BiggestLoserPL     = PairOverallPL;
          BiggestLoserLots   = OrderLots();
          BiggestLoserTicket = OrderTicket();
          BiggestLoserType   = "SELL";
          BiggestLoserSymbol = OrderSymbol();
        }

        SellLots += OrderLots();

        NumSells++;

        if (OrderSymbol() == GBPUSDsym) { GBPUSDsells++; GBPUSDPL = PairOverallPL; }
        else
        if (OrderSymbol() == EURGBPsym) { EURGBPsells++; EURGBPPL = PairOverallPL; }
        else
        if (OrderSymbol() == GBPJPYsym) { GBPJPYsells++; GBPJPYPL = PairOverallPL; }
        else
        if (OrderSymbol() == USDCHFsym) { USDCHFsells++; USDCHFPL = PairOverallPL; }
        else
        if (OrderSymbol() == NZDUSDsym) { NZDUSDsells++; NZDUSDPL = PairOverallPL; }
        else
        if (OrderSymbol() == AUDJPYsym) { AUDJPYsells++; AUDJPYPL = PairOverallPL; }
        else
        if (OrderSymbol() == EURJPYsym) { EURJPYsells++; EURJPYPL = PairOverallPL; }
        else
        if (OrderSymbol() == EURUSDsym) { EURUSDsells++; EURUSDPL = PairOverallPL; }
        else
        if (OrderSymbol() == USDJPYsym) { USDJPYsells++; USDJPYPL = PairOverallPL; }
        else
        if (OrderSymbol() == AUDUSDsym) { AUDUSDsells++; AUDUSDPL = PairOverallPL; }
        else
        if (OrderSymbol() == NZDJPYsym) { NZDJPYsells++; NZDJPYPL = PairOverallPL; }
        else
        if (OrderSymbol() == GBPCHFsym) { GBPCHFsells++; GBPCHFPL = PairOverallPL; }
        else
        if (OrderSymbol() == CHFJPYsym) { CHFJPYsells++; CHFJPYPL = PairOverallPL; }
        else
        if (OrderSymbol() == EURCHFsym) { EURCHFsells++; EURCHFPL = PairOverallPL; }
      }
    }
  }
  TotalTrades = NumBuys+NumSells;

  return(0);
}

//+------------------------------------------------------------------+
//|         Set the dollar profit and trim target values.            |
//+------------------------------------------------------------------+

void SetExitTargets()
{
  DollarProfitTarget = StrToDouble(DoubleToStr(valLastFlatEq*ProfitPct/100.0,2));

  if (AccountIsMini)
  {
    if (DollarProfitTarget > MaxMiniProfit) DollarProfitTarget = MaxMiniProfit;
  }
  else
  {
    if (DollarProfitTarget > MaxStdProfit) DollarProfitTarget = MaxStdProfit;
  }

  return(0);
}

//+------------------------------------------------------------------+
//|                 Close all of the open trades.                    |
//+------------------------------------------------------------------+

void CloseAllTradesIfNeeded()
{
  if (BidsHaveChanged && CloseAll)
  {
    for(i = 0; i < OrdersTotal(); i++)
    {
      OrderSelect(i,SELECT_BY_POS,MODE_TRADES);

      if (OrderMagicNumber() == MagicNumber)
      {
        if (OrderType() == OP_BUY)
        {
          Comment("In full closure mode.  Closing a ", OrderSymbol(), " buy trade...");
          Print("In full closure mode.  Closing a ", OrderSymbol(), " buy trade...");
          OrderClose(OrderTicket(),OrderLots(),MarketInfo(OrderSymbol(), MODE_BID),Slippage,White);
          Sleep(1000);
        }    
        else
        if (OrderType() == OP_SELL)
        {
          Comment("In full closure mode.  Closing a ", OrderSymbol(), " sell trade...");
          Print("In full closure mode.  Closing a ", OrderSymbol(), " sell trade...");
          OrderClose(OrderTicket(),OrderLots(),MarketInfo(OrderSymbol(), MODE_ASK),Slippage,White);
          Sleep(1000);
        }
      }
    }
  }
  
  return(0);
}

//+------------------------------------------------------------------+
//| Track the number of pairs that have risen/fallen above/below     |
//| their 24-hour highs/lows.                                        |
//+------------------------------------------------------------------+

void CheckExceedences()
{
  GBPUSDrose = false;
  GBPUSDsank = false;
  EURGBProse = false;
  EURGBPsank = false;
  GBPJPYrose = false;
  GBPJPYsank = false;
  USDCHFrose = false;
  USDCHFsank = false;
  NZDUSDrose = false;
  NZDUSDsank = false;
  AUDJPYrose = false;
  AUDJPYsank = false;
  EURJPYrose = false;
  EURJPYsank = false;

  EURUSDrose = false;
  EURUSDsank = false;
  USDJPYrose = false;
  USDJPYsank = false;
  AUDUSDrose = false;
  AUDUSDsank = false;
  NZDJPYrose = false;
  NZDJPYsank = false;
  GBPCHFrose = false;
  GBPCHFsank = false;
  CHFJPYrose = false;
  CHFJPYsank = false;
  EURCHFrose = false;
  EURCHFsank = false;

  if (MarketInfo(GBPUSDsym, MODE_BID) <= iLow(GBPUSDsym, PERIOD_H1, iLowest(GBPUSDsym, PERIOD_H1, MODE_LOW, 24, 0)))
    GBPUSDsank = true;
  else
  if (MarketInfo(GBPUSDsym, MODE_ASK) >= iHigh(GBPUSDsym, PERIOD_H1, iHighest(GBPUSDsym, PERIOD_H1, MODE_HIGH, 24, 0)))
    GBPUSDrose = true;

  if (MarketInfo(EURGBPsym, MODE_BID) <= iLow(EURGBPsym, PERIOD_H1, iLowest(EURGBPsym, PERIOD_H1, MODE_LOW, 24, 0)))
    EURGBPsank = true;
  else
  if (MarketInfo(EURGBPsym, MODE_ASK) >= iHigh(EURGBPsym, PERIOD_H1, iHighest(EURGBPsym, PERIOD_H1, MODE_HIGH, 24, 0)))
    EURGBProse = true;

  if (MarketInfo(GBPJPYsym, MODE_BID) <= iLow(GBPJPYsym, PERIOD_H1, iLowest(GBPJPYsym, PERIOD_H1, MODE_LOW, 24, 0)))
    GBPJPYsank = true;
  else
  if (MarketInfo(GBPJPYsym, MODE_ASK) >= iHigh(GBPJPYsym, PERIOD_H1, iHighest(GBPJPYsym, PERIOD_H1, MODE_HIGH, 24, 0)))
    GBPJPYrose = true;

  if (MarketInfo(USDCHFsym, MODE_BID) <= iLow(USDCHFsym, PERIOD_H1, iLowest(USDCHFsym, PERIOD_H1, MODE_LOW, 24, 0)))
    USDCHFsank = true;
  else
  if (MarketInfo(USDCHFsym, MODE_ASK) >= iHigh(USDCHFsym, PERIOD_H1, iHighest(USDCHFsym, PERIOD_H1, MODE_HIGH, 24, 0)))
    USDCHFrose = true;

  if (MarketInfo(NZDUSDsym, MODE_BID) <= iLow(NZDUSDsym, PERIOD_H1, iLowest(NZDUSDsym, PERIOD_H1, MODE_LOW, 24, 0)))
    NZDUSDsank = true;
  else
  if (MarketInfo(NZDUSDsym, MODE_ASK) >= iHigh(NZDUSDsym, PERIOD_H1, iHighest(NZDUSDsym, PERIOD_H1, MODE_HIGH, 24, 0)))
    NZDUSDrose = true;

  if (MarketInfo(AUDJPYsym, MODE_BID) <= iLow(AUDJPYsym, PERIOD_H1, iLowest(AUDJPYsym, PERIOD_H1, MODE_LOW, 24, 0)))
    AUDJPYsank = true;
  else
  if (MarketInfo(AUDJPYsym, MODE_ASK) >= iHigh(AUDJPYsym, PERIOD_H1, iHighest(AUDJPYsym, PERIOD_H1, MODE_HIGH, 24, 0)))
    AUDJPYrose = true;

  if (MarketInfo(EURJPYsym, MODE_BID) <= iLow(EURJPYsym, PERIOD_H1, iLowest(EURJPYsym, PERIOD_H1, MODE_LOW, 24, 0)))
    EURJPYsank = true;
  else
  if (MarketInfo(EURJPYsym, MODE_ASK) >= iHigh(EURJPYsym, PERIOD_H1, iHighest(EURJPYsym, PERIOD_H1, MODE_HIGH, 24, 0)))
    EURJPYrose = true;

  if (MarketInfo( EURUSDsym, MODE_BID) <= iLow( EURUSDsym, PERIOD_H1, iLowest( EURUSDsym, PERIOD_H1, MODE_LOW, 24, 0)))
     EURUSDsank = true;
  else
  if (MarketInfo( EURUSDsym, MODE_ASK) >= iHigh( EURUSDsym, PERIOD_H1, iHighest( EURUSDsym, PERIOD_H1, MODE_HIGH, 24, 0)))
     EURUSDrose = true;

  if (MarketInfo(USDJPYsym, MODE_BID) <= iLow(USDJPYsym, PERIOD_H1, iLowest(USDJPYsym, PERIOD_H1, MODE_LOW, 24, 0)))
    USDJPYsank = true;
  else
  if (MarketInfo(USDJPYsym, MODE_ASK) >= iHigh(USDJPYsym, PERIOD_H1, iHighest(USDJPYsym, PERIOD_H1, MODE_HIGH, 24, 0)))
    USDJPYrose = true;

  if (MarketInfo(AUDUSDsym, MODE_BID) <= iLow(AUDUSDsym, PERIOD_H1, iLowest(AUDUSDsym, PERIOD_H1, MODE_LOW, 24, 0)))
    AUDUSDsank = true;
  else
  if (MarketInfo(AUDUSDsym, MODE_ASK) >= iHigh(AUDUSDsym, PERIOD_H1, iHighest(AUDUSDsym, PERIOD_H1, MODE_HIGH, 24, 0)))
    AUDUSDrose = true;

  if (MarketInfo(NZDJPYsym, MODE_BID) <= iLow(NZDJPYsym, PERIOD_H1, iLowest(NZDJPYsym, PERIOD_H1, MODE_LOW, 24, 0)))
    NZDJPYsank = true;
  else
  if (MarketInfo(NZDJPYsym, MODE_ASK) >= iHigh(NZDJPYsym, PERIOD_H1, iHighest(NZDJPYsym, PERIOD_H1, MODE_HIGH, 24, 0)))
    NZDJPYrose = true;

  if (MarketInfo(GBPCHFsym, MODE_BID) <= iLow(GBPCHFsym, PERIOD_H1, iLowest(GBPCHFsym, PERIOD_H1, MODE_LOW, 24, 0)))
    GBPCHFsank = true;
  else
  if (MarketInfo(GBPCHFsym, MODE_ASK) >= iHigh(GBPCHFsym, PERIOD_H1, iHighest(GBPCHFsym, PERIOD_H1, MODE_HIGH, 24, 0)))
    GBPCHFrose = true;

  if (MarketInfo(CHFJPYsym, MODE_BID) <= iLow(CHFJPYsym, PERIOD_H1, iLowest(CHFJPYsym, PERIOD_H1, MODE_LOW, 24, 0)))
    CHFJPYsank = true;
  else
  if (MarketInfo(CHFJPYsym, MODE_ASK) >= iHigh(CHFJPYsym, PERIOD_H1, iHighest(CHFJPYsym, PERIOD_H1, MODE_HIGH, 24, 0)))
    CHFJPYrose = true;

  if (MarketInfo(EURCHFsym, MODE_BID) <= iLow(EURCHFsym, PERIOD_H1, iLowest(EURCHFsym, PERIOD_H1, MODE_LOW, 24, 0)))
    EURCHFsank = true;
  else
  if (MarketInfo(EURCHFsym, MODE_ASK) >= iHigh(EURCHFsym, PERIOD_H1, iHighest(EURCHFsym, PERIOD_H1, MODE_HIGH, 24, 0)))
    EURCHFrose = true;

  return(0);
}

//+------------------------------------------------------------------+
//|                     Trim a biggest loser.                        |
//+------------------------------------------------------------------+

void TrimIfNeeded()
{
  bool CloseResult;

  CheckExceedences();

  if (BidsHaveChanged &&
      MathAbs(BiggestLoserPL) > DollarProfitTarget &&
     !CloseAll)
  {
    for (i = 0; i < OrdersTotal(); i++)
    {
      OrderSelect(i,SELECT_BY_POS,MODE_TRADES);

      if (OrderMagicNumber() == MagicNumber &&
          BiggestLoserTicket == OrderTicket())
      {
        if (OrderType() == OP_BUY &&
            MarketInfo(BiggestLoserSymbol, MODE_BID) <= iLow(BiggestLoserSymbol, PERIOD_H1, iLowest(BiggestLoserSymbol, PERIOD_H1, MODE_LOW, 24, 0)))
        {
          if (!((OrderSymbol() == GBPUSDsym && (GBPJPYsank || GBPCHFsank || EURGBProse)) ||
                (OrderSymbol() == EURGBPsym && (EURJPYsank || EURCHFsank)) ||
                (OrderSymbol() == GBPJPYsym && (EURGBProse || GBPCHFsank)) ||
                (OrderSymbol() == USDCHFsym && (EURCHFsank || GBPCHFsank)) ||
                (OrderSymbol() == NZDUSDsym &&  NZDJPYsank) ||
                (OrderSymbol() == AUDJPYsym && (CHFJPYsank || EURJPYsank)) ||
                (OrderSymbol() == EURJPYsym && (EURGBPsank || EURCHFsank)) ||

                (OrderSymbol() == EURUSDsym && (EURJPYsank || EURGBPsank && EURCHFsank)) ||
                (OrderSymbol() == USDJPYsym && (CHFJPYsank || NZDJPYsank)) ||
                (OrderSymbol() == AUDUSDsym &&  AUDJPYsank) ||
                (OrderSymbol() == NZDJPYsym && (CHFJPYsank || EURJPYsank)) ||
                (OrderSymbol() == GBPCHFsym && (EURGBProse || GBPJPYsank)) ||
                (OrderSymbol() == CHFJPYsym && (EURCHFrose || GBPCHFrose)) ||
                (OrderSymbol() == EURCHFsym && (EURJPYsank || EURGBPsank))))
          {
            return(0);
          }

          Comment("Closing a " + BiggestLoserSymbol + " biggest-loser buy trade...");
          Print("Closing a " + BiggestLoserSymbol + " biggest-loser buy trade...");
          CloseResult = OrderClose(BiggestLoserTicket,OrderLots(),MarketInfo(BiggestLoserSymbol, MODE_BID),Slippage,White);
          Sleep(1000);

          if (CloseResult) // If closure succeeded, open the same symbol as a SELL
          {
            Comment("Opening a " + BiggestLoserSymbol + " sell trade...");
            Print("Short " + BiggestLoserSymbol + ".");
            OrderSend(BiggestLoserSymbol,OP_SELL,UseLots,MarketInfo(BiggestLoserSymbol, MODE_BID),Slippage,0,0,"Biggest loser SOLD",MagicNumber,Red);
            Sleep(1000);
          }
        }
        else
        if (OrderType() == OP_SELL &&
            MarketInfo(BiggestLoserSymbol, MODE_ASK) >= iHigh(BiggestLoserSymbol, PERIOD_H1, iHighest(BiggestLoserSymbol, PERIOD_H1, MODE_HIGH, 24, 0)))
        {
          if (!((OrderSymbol() == GBPUSDsym && (GBPJPYrose || GBPCHFrose || EURGBPsank)) ||
                (OrderSymbol() == EURGBPsym && (EURJPYrose || EURCHFrose)) ||
                (OrderSymbol() == GBPJPYsym && (EURGBPsank || GBPCHFrose)) ||
                (OrderSymbol() == USDCHFsym && (EURCHFrose || GBPCHFrose)) ||
                (OrderSymbol() == NZDUSDsym &&  NZDJPYrose) ||
                (OrderSymbol() == AUDJPYsym && (CHFJPYrose || EURJPYrose)) ||
                (OrderSymbol() == EURJPYsym && (EURGBProse || EURCHFrose)) ||

                (OrderSymbol() == EURUSDsym && (EURJPYrose || EURGBProse || EURCHFrose)) ||
                (OrderSymbol() == USDJPYsym && (CHFJPYrose || NZDJPYrose)) ||
                (OrderSymbol() == AUDUSDsym && AUDJPYrose) ||
                (OrderSymbol() == NZDJPYsym && (CHFJPYrose || EURJPYrose)) ||
                (OrderSymbol() == GBPCHFsym && (EURGBPsank || GBPJPYrose)) ||
                (OrderSymbol() == CHFJPYsym && (EURCHFsank || GBPCHFsank)) ||
                (OrderSymbol() == EURCHFsym && (EURJPYrose || EURGBProse))))
          {
            return(0);
          }

          Comment("Closing a " + BiggestLoserSymbol + " biggest-loser sell trade...");
          Print("Closing a " + BiggestLoserSymbol + " biggest-loser sell trade...");
          CloseResult = OrderClose(BiggestLoserTicket,OrderLots(),MarketInfo(BiggestLoserSymbol, MODE_ASK),Slippage,White);
          Sleep(1000);

          if (CloseResult) // If closure succeeded, open the same symbol as a BUY
          {
            Comment("Opening a " + BiggestLoserSymbol + " buy trade...");
            Print("Long " + BiggestWinnerSymbol + ".");
            OrderSend(BiggestLoserSymbol,OP_BUY,UseLots,MarketInfo(BiggestLoserSymbol, MODE_ASK),Slippage,0,0,"Biggest loser BOUGHT",MagicNumber,Blue);
            Sleep(1000);
          }
        }
      }
    } 
  }

  return(0);
}

//+------------------------------------------------------------------+
//|                         Open Trades                              |
//+------------------------------------------------------------------+

void OpenTradesIfNeeded()
{
  if (BidsHaveChanged && !CloseAll && TotalTrades < MaxTrades)
  {
    // Go long on a buy pair there is no trade open for it.
      
    if (GBPUSDbuys == 0 && GBPUSDsells == 0)
    {
      Comment("Opening a buy trade...");
      Print("Long GBPUSD.");
      OrderSend(GBPUSDsym,OP_BUY,UseLots,MarketInfo(GBPUSDsym, MODE_ASK),Slippage,0,0,"Buy Slot BOUGHT",MagicNumber,Blue);
      Sleep(1000);
    }

    if (EURGBPbuys == 0 && EURGBPsells == 0)
    {
      Comment("Opening a buy trade...");
      Print("Long EURGBP.");
      OrderSend(EURGBPsym,OP_BUY,UseLots,MarketInfo(EURGBPsym, MODE_ASK),Slippage,0,0,"Buy Slot BOUGHT",MagicNumber,Blue);
      Sleep(1000);
    }

    if (GBPJPYbuys == 0 && GBPJPYsells == 0)
    {
      Comment("Opening a buy trade...");
      Print("Long GBPJPY.");
      OrderSend(GBPJPYsym,OP_BUY,UseLots,MarketInfo(GBPJPYsym, MODE_ASK),Slippage,0,0,"Buy Slot BOUGHT",MagicNumber,Blue);
      Sleep(1000);
    }

    if (USDCHFbuys == 0 && USDCHFsells == 0)
    {
      Comment("Opening a buy trade...");
      Print("Long USDCHF.");
      OrderSend(USDCHFsym,OP_BUY,UseLots,MarketInfo(USDCHFsym, MODE_ASK),Slippage,0,0,"Buy Slot BOUGHT",MagicNumber,Blue);
      Sleep(1000);
    }

    if (NZDUSDbuys == 0 && NZDUSDsells == 0)
    {
      Comment("Opening a buy trade...");
      Print("Long NZDUSD.");
      OrderSend(NZDUSDsym,OP_BUY,UseLots,MarketInfo(NZDUSDsym, MODE_ASK),Slippage,0,0,"Buy Slot BOUGHT",MagicNumber,Blue);
      Sleep(1000);
    }

    if (AUDJPYbuys == 0 && AUDJPYsells == 0)
    {
      Comment("Opening a buy trade...");
      Print("Long AUDJPY.");
      OrderSend(AUDJPYsym,OP_BUY,UseLots,MarketInfo(AUDJPYsym, MODE_ASK),Slippage,0,0,"Buy Slot BOUGHT",MagicNumber,Blue);
      Sleep(1000);
    }

    if (EURJPYbuys == 0 && EURJPYsells == 0)
    {
      Comment("Opening a buy trade...");
      Print("Long EURJPY.");
      OrderSend(EURJPYsym,OP_BUY,UseLots,MarketInfo(EURJPYsym, MODE_ASK),Slippage,0,0,"Buy Slot BOUGHT",MagicNumber,Blue);
      Sleep(1000);
    }

    // Go short on a sell pair if there is no trade open for it.
      
    if (EURUSDsells == 0 && EURUSDbuys == 0)
    {
      Comment("Opening a sell trade...");
      Print("Short EURUSD.");
      OrderSend(EURUSDsym,OP_SELL,UseLots,MarketInfo(EURUSDsym, MODE_BID),Slippage,0,0,"Sell Slot SOLD",MagicNumber,Red);
      Sleep(1000);
    }

    if (USDJPYsells == 0 && USDJPYbuys == 0)
    {
      Comment("Opening a sell trade...");
      Print("Short USDJPY.");
      OrderSend(USDJPYsym,OP_SELL,UseLots,MarketInfo(USDJPYsym, MODE_BID),Slippage,0,0,"Sell Slot SOLD",MagicNumber,Red);
      Sleep(1000);
    }

    if (AUDUSDsells == 0 && AUDUSDbuys == 0)
    {
      Comment("Opening a sell trade...");
      Print("Short AUDUSD.");
      OrderSend(AUDUSDsym,OP_SELL,UseLots,MarketInfo(AUDUSDsym, MODE_BID),Slippage,0,0,"Sell Slot SOLD",MagicNumber,Red);
      Sleep(1000);
    }

    if (NZDJPYsells == 0 && NZDJPYbuys == 0)
    {
      Comment("Opening a sell trade...");
      Print("Short NZDJPY.");
      OrderSend(NZDJPYsym,OP_SELL,UseLots,MarketInfo(NZDJPYsym, MODE_BID),Slippage,0,0,"Sell Slot SOLD",MagicNumber,Red);
      Sleep(1000);
    }

    if (GBPCHFsells == 0 && GBPCHFbuys == 0)
    {
      Comment("Opening a sell trade...");
      Print("Short GBPCHF.");
      OrderSend(GBPCHFsym,OP_SELL,UseLots,MarketInfo(GBPCHFsym, MODE_BID),Slippage,0,0,"Sell Slot SOLD",MagicNumber,Red);
      Sleep(1000);
    }

    if (CHFJPYsells == 0 && CHFJPYbuys == 0)
    {
      Comment("Opening a sell trade...");
      Print("Short CHFJPY.");
      OrderSend(CHFJPYsym,OP_SELL,UseLots,MarketInfo(CHFJPYsym, MODE_BID),Slippage,0,0,"Sell Slot SOLD",MagicNumber,Red);
      Sleep(1000);
    }

    if (EURCHFsells == 0 && EURCHFbuys == 0)
    {
      Comment("Opening a sell trade...");
      Print("Short EURCHF.");
      OrderSend(EURCHFsym,OP_SELL,UseLots,MarketInfo(EURCHFsym, MODE_BID),Slippage,0,0,"Sell Slot SOLD",MagicNumber,Red);
      Sleep(1000);
    }
  }

  return(0);
}

//+------------------------------------------------------------------+
//|   Keep track of lowest margin level and largest floating loss.   |
//+------------------------------------------------------------------+

void ExamineMarginAndFloat()
{
  if (AccountMargin() > 0.0)
  {
    PercentMarginLevel = StrToDouble(DoubleToStr(100 * AccountEquity() / AccountMargin(), 2));

    if (PercentMarginLevel < valLowestMarginLevel)
    {
      valLowestMarginLevel = PercentMarginLevel;
      GlobalVariableSet(LowestMarginLevel, valLowestMarginLevel);
    }
  }

  if (EAPL < valLargestFloatingLoss)
  {
    valLargestFloatingLoss = StrToDouble(DoubleToStr(EAPL, 2));
    GlobalVariableSet(LargestFloatingLoss, valLargestFloatingLoss);
  }

  if ((valLargestFloatingLoss / valLastFlatEq) * 100.0 < valMaxFloatingDrawdown)
  {
    valMaxFloatingDrawdown = StrToDouble(DoubleToStr((valLargestFloatingLoss / valLastFlatEq) * 100.0, 2));
    GlobalVariableSet(MaxFloatingDrawdown, valMaxFloatingDrawdown);
  }

  return(0);
}

//+------------------------------------------------------------------+
//|                      Print Info to Chart                         |
//+------------------------------------------------------------------+

void PrintChartComments()
{
  CommentString = 
        "STRATEGY: Trim every biggest loser that rises/falls above/below the 24-hour high/low for that currency pair if" +
        "\none of the confirming pairs have also risen/fallen above/below their 24-hour highs/lows, and place a trade in the opposite direction." +
        "\nREMINDER: Be FLAT (all trades closed) before depositing/withdrawing funds." +
        "\nAccount Type: " + AccountTypeString + " Account,   Leverage: " + DoubleToStr(Leverage,0) + ":1" +
        ",   Margin %: " + DoubleToStr(PercentMarginLevel,2) + "%,   Lowest Margin %: " + DoubleToStr(valLowestMarginLevel,2) + "%" +
        "\nLargest Floating Loss: $" + DoubleToStr(valLargestFloatingLoss,2) + ",  Max Floating Drawdown %: " + DoubleToStr(valMaxFloatingDrawdown,2) + "%" +
        "\nBalance: $" + DoubleToStr(AccountBalance(),2) + ",   Equity: $" + DoubleToStr(AccountEquity(),2) +
        "\nLast Flat Balance: $" + DoubleToStr(valLastFlatBal,2) + ",   Last Flat Equity: $" + DoubleToStr(valLastFlatEq,2) +
        "\nDollar Profit Target: $" + DoubleToStr(DollarProfitTarget,2) + ",   Equity Profit Target: $" + DoubleToStr(valLastFlatEq+DollarProfitTarget,2) +
        "\nOpen Buys: " + DoubleToStr(NumBuys,0) + ",   Open Sells: " + DoubleToStr(NumSells,0) + ",   Total Open Trades: " + DoubleToStr(TotalTrades,0) +
        "\nEA PL: $" + DoubleToStr(EAPL,2) + ",   Buy PL: $" + DoubleToStr(BuyPL,2) + ",   Sell PL: $" + DoubleToStr(SellPL,2) +
        "\nBiggest Winner Symbol: " + BiggestWinnerSymbol + ",   Biggest Winner Type: " + BiggestWinnerType + ",  Biggest Winner Ticket: " + DoubleToStr(BiggestWinnerTicket,0) + ",   Biggest Winner PL: $" + DoubleToStr(BiggestWinnerPL,2) +
        "\nBiggest Loser Symbol: " + BiggestLoserSymbol + ",   Biggest Loser Type: " + BiggestLoserType + ",  Biggest Loser Ticket: " + DoubleToStr(BiggestLoserTicket,0) + ",   Biggest Loser PL: $" + DoubleToStr(BiggestLoserPL,2);
        
  Comment(CommentString);

  return(0);
}